#ifndef __QT4CUPS_H
#define __QT4CUPS_H

#include <QObject>
#include <QStringList>
#include <QVariant>
#include <QDateTime>
#include <QByteArray>
#include <QTextCodec>
#include <cups/cups.h>
#include <cups/http.h>
#include <cups/ipp.h>
#include <Ipp.h>
#include <fly-admin-printer_export.h>

namespace Cups
{
extern FLY_ADMIN_PRINTER_EXPORT QTextCodec *codec;

void dumpAttributes(const QVariantMap &attrs);

////////////////////////////////////////////////////////////////////////////////
struct FLY_ADMIN_PRINTER_EXPORT Job
{
    int id;
    QString dest;
    QString title;
    QString user;
    QString format;
    ipp_jstate_t state;
    int size;
    int priority;
    QDateTime completed_time;
    QDateTime creation_time;
    QDateTime processing_time;
    Job(const cups_job_t &job)
    {
        id = job.id;
        if (job.dest) dest = codec->toUnicode(job.dest);
        if (job.title) title = codec->toUnicode(job.title);
        if (job.user) user = codec->toUnicode(job.user);
        if (job.format) format = codec->toUnicode(job.format);
        state = job.state;
        size = job.size;
        priority = job.priority;
        completed_time = QDateTime::fromTime_t(job.completed_time);
        creation_time = QDateTime::fromTime_t(job.creation_time);
        processing_time = QDateTime::fromTime_t(job.processing_time);
    }
};


////////////////////////////////////////////////////////////////////////////////

//int cupsAdminExportSamba(const QString &dest, const QString &ppd,
//    const QString &samba_server, const QString &samba_user,
//    const QString &samba_password, const QString &logfile=QString());

////////////////////////////////////////////////////////////////////////////////

QString statusName(ipp_status_t);
QString tagName(ipp_tag_t);
QString opName(ipp_op_t);
QString orientName(ipp_orient_t);

class FLY_ADMIN_PRINTER_EXPORT HTTP: public QObject
{
    Q_OBJECT
private:
    http_t *http_;
private:
    HTTP(http_t *http);

    static HTTP* create(const QString &host, int port, http_encryption_t encryption);
    static HTTP* create();

    ipp_pstate_t destState(const QString &uri);

public:
    enum JobCompletion
    {
        All = -1, Active = 0, Completed = 1
    };

    http_t *handle() const;

    ~HTTP();

    void reconnect();
    //constructor
    static HTTP* getUniqueConnection();

    QMap <QString, QStringList> cupsAdminGetServerSettings();
    bool isNull() const;
    QList <QVariantMap> destinations();
    QVariantMap destAttrs(const QString &uri);
    bool setPrinterPolicy(const QString &uri, const QString &policy);
    bool setPrinterMac(const QString &uri, const QString &min, const QString &max);
    QList < QMap <QString, QString> > cupsDestinations(const QString &name = "");
    QList <Job> jobs(const QString &dest = QString::null, bool myjobs = false, JobCompletion completed = Active);
    QStringList backends();
    QList <QVariantMap> devices();
    QList <QVariantMap> PPDs();
    ipp_pstate_t printerState(const QString &name);
    ipp_pstate_t classState(const QString &name);
    bool enablePrinter(const QString &uri);
    bool disablePrinter(const QString &uri);
    bool printerSetServerDefault(const QString &uri);
    bool addPrinter(const QString &uri, const QString &name, const QString &ppd, const QString &location = QString::null, const QString &description = QString::null, bool shared = false);
    bool addClass(const QString &name, const QStringList &uris, const QString &location = QString::null, const QString &description = QString::null, bool shared = false);
    bool deleteClass(const QString &uri);
    bool deletePrinter(const QString &uri);
    bool holdJob(const QString &uri, int id);
    bool releaseJob(const QString &uri, int id);
    bool cancelJob(const QString &dest, int id);
    bool purgeJob(const QString &dest, int id);
    bool restartJob(const QString &dest, int id);
signals:
    void step(int steps);
};

} //namespace

#endif//__QT4CUPS_H
