#ifndef IPP_H
#define IPP_H

#include <QObject>
#include <QStringList>
#include <QVariant>
#include <QDateTime>
#include <QByteArray>
#include <QTextCodec>
#include <cups/cups.h>
#include <cups/http.h>
#include <cups/ipp.h>
#include <fly-admin-printer_export.h>

namespace Cups {

class HTTP;

////////////////////////////////////////////////////////////////////////////////
class FLY_ADMIN_PRINTER_EXPORT IPP : public QObject
{
    Q_OBJECT
public:
    class Value
    {
    public:
        Value(ipp_attribute_t *attribute, int element);

        struct Resolution
        {

            Resolution();
            Resolution(int w, int h, ipp_res_t u);
            Resolution(const QVariant &src);
            operator QVariant() const;

            int width, height;
            ipp_res_t units;

        };
        struct Range
        {
            Range(int min, int max);
            Range(const QVariant &src);
            bool isNull() const;
            operator QVariant() const;
            int min = 0, max = 0;
        };
        int asInt() const;
        bool asBool() const;
        QDateTime asDateTime() const;
        Resolution asResolution() const;
        Range asRange() const;
        QString asString() const;
        QVariant convert() const;

    private:
        ipp_attribute_t *attribute_;
        int element_;
    };

    enum PrinterType
    {
        Class = 0x00000001,     //Is a printer class.
        Remote = 0x00000002,        //Is a remote destination.
        Black = 0x00000004,     //Can print in black.
        Color = 0x00000008,     //Can print in color.
        Double = 0x00000010,        //Can print on both sides of the page in hardware.
        Staple = 0x00000020,        //Can staple output.
        Copy = 0x00000040,      //Can do fast copies in hardware.
        Collation = 0x00000080,     //Can do fast copy collation in hardware.
        Punch = 0x00000100,     //Can punch output.
        Cover = 0x00000200,     //Can cover output.
        Bind = 0x00000400,      //Can bind output.
        Sort = 0x00000800,      //Can sort output.
        Small = 0x00001000,     //Can handle media up to US-Legal/A4.
        Medium = 0x00002000,        //Can handle media from US-Legal/A4 to ISO-C/A2.
        Large = 0x00004000,     //Can handle media larger than ISO-C/A2.
        Custom = 0x00008000,        //Can handle user-defined media sizes.
        Implicit = 0x00010000,      //Is an implicit (server-generated) class.
        Default = 0x00020000,       //Is the a default printer on the network.
        Fax = 0x00040000,           //Is a facsimile device.
        Reject = 0x00080000,        //Is rejecting jobs.
        Delete = 0x00100000,        //Delete this queue.
        NotShared = 0x00200000,     //Queue is not shared.
        ReqAuth = 0x00400000,       //Queue requires authentication.
        Command = 0x00800000,       //Queue supports CUPS command files.
        AutoDiscovery = 0x01000000  //Queue was automatically discovered and added.
    };

    Q_DECLARE_FLAGS (PrinterTypeFlags, PrinterType)

    static QString typeName(PrinterType type);
    static IPP* create(ipp_op_t op);
    static IPP* create();

    ~IPP();

    ipp_t *handle();
    bool isNull() const;

    ipp_state_t state() const;
    QString stateName() const;
    void setState(ipp_state_t state);

    ipp_attribute_t *firstAttribute() const;
    ipp_attribute_t *nextAttribute() const;
    QList <QVariantMap> attributes() const;
    QMap <QString, ipp_tag_t> valueTags() const;
    QMap <QString, ipp_tag_t> groupTags() const;

    void addAttribute(const QString &name, ipp_tag_t tag, const QStringList &values);
    void addAttribute(const QString &name, ipp_tag_t tag, const QString &value);
    void addAttribute(const QString &name, ipp_tag_t tag, int numValues, const char * const *values);
    void addPrinterAttribute(const QString &name, ipp_tag_t tag, const QString &value);
    void addJobAttribute(const QString &name, ipp_tag_t tag, const QString &value);
    void addJobAttribute(const QString &name, int value);
    void addAttribute(const QString &name, int value);
    void addAttribute(ipp_tag_t tag, const QString &name, int value);
    void addAttribute(const QString &name, bool value);
    void setOperation(ipp_op_t op);
    IPP *doRequest(HTTP *connection, const QString &resource);
    IPP *doFileRequest(HTTP *connection, const QString &resource, const QString &fileName);

protected:
    IPP(ipp_t * /*ipp*/ );

private:
     ipp_t *ipp_;

};

} //namespace

#endif // IPP_H
