#ifndef __JOBWIDGET_H
#define __JOBWIDGET_H

#include <QTreeWidget>
#include <QPointer>
#include <fly-admin-printer_export.h>

class QAction;
class QEvent;
class QTimer;
class ISecurityJobAction;

namespace Cups
{
class HTTP;
}

class FLY_ADMIN_PRINTER_EXPORT JobWidget: public QTreeWidget
{
    Q_OBJECT
public:
    JobWidget(QWidget *parent = nullptr);
    ~JobWidget();
    void load(QPointer<Cups::HTTP> connection, const QString &printer = QString());

    QAction *m_aPlay;
    QAction *m_aAbort;
    QAction *m_aRestart;
    QAction *m_aMove;
    QAction *m_aPurge;
    QAction *m_aShowCompleted;
    QAction *m_aShowCanceled;
    QAction *m_aShowAborted;

public slots:
    virtual void currentItemChanged(QTreeWidgetItem*it, QTreeWidgetItem*);
    virtual void reload();
    virtual void cancelJob();
    virtual void purgeJob();
    virtual void playJob();
    virtual void restartJob();

protected:
    virtual bool eventFilter(QObject *o, QEvent *e) override;
    void connectToDBus();
    void disconnectFromDBus();

protected slots:
    void jobQueuedLocal(const QString&, quint32, const QString&);
    void jobStartedLocal(const QString&, quint32, const QString&);

signals:
    void jobsStateChanged();

private:
    QTimer *m_refreshTimer;
    QList<ISecurityJobAction*> m_extraActions;
    QPointer<Cups::HTTP> m_connection;
    QString m_printer;
};

#endif
